# -*- coding: utf-8 -*-

"""
Módulo que oferece funcionalidades para permitir a definição e aplicação de cache em URLs de APIs Flask
"""


def definicao(nome_api=None):
    """
    Método construtor para que uma app inicialize um cache sem o risco de misturar com outra app
    :param nome_api: O nome da api
    :type nome_api: str
    :return: Um objeto para montar as definições de cache
    :rtype cache_api.DefinicaoDeCache
    """
    return DefinicaoDeCache(nome_api)


class DefinicaoDeCache(object):
    """
    Classe que define os caches e permite aplica-los no cabeçalho http Cache-Control da reposta
    """

    def __init__(self, nome_api=None):
        self.nome_api = nome_api
        self.definicoes = {}
        self.regras_sem_cache = {}

    def define_cache(self, url, max_age=0, sem_cache=None):
        """
        Define os dados de cache para uma URL
        :param url: A URL para qual o cache será definido
        :type url: str
        :param max_age: O tempo de vida, em segundos, do cache
        :type max_age: int
        :param sem_cache: Dicionário para definição de regras para que o cache não seja aplicado. Ex: sem_cache={'querystring': 'no-cache=1'} define que
        quando a url for chamada com uma querystring contendo no-cache=1, o cache não será aplicado.
        :type sem_cache: dict
        :return: A url para ser usada na definição da rota
        :rtype: str
        """
        self.definicoes[url] = {
            'max_age': max_age
        }
        if sem_cache is not None:
            self.regras_sem_cache[url] = sem_cache
        return url

    def aplica_cache(self, request, headers, **excecoes):
        """
        Aplica um cache na URL que doi acessada
        :param request: O objeto request da requisição Flask
        :type request: flask.request
        :param headers: Um dicionário onde a chave Cache-Control será adicionada
        :type headers: dict
        :param excecoes: Parâmetros que podem definir exceções à aplicação de cache. Caso um parâmetro seja passado no modo param=False e o
        mesmo parâmetro tenha sido registrado no DefinicaoDeCache.define_cache, o cache não será aplicado
        :type excecoes: kwargs
        :return: None
        """
        url = request.url_rule.rule
        if url in self.definicoes:
            max_age = self.definicoes[url]['max_age']
            regra = self.regras_sem_cache.get(url, {})
            if regra:
                querystring = regra.get('querystring', None)
                if querystring and querystring in request.query_string:
                    max_age = 0
                parametro = regra.get('parametro', '')
                excecao = excecoes.get(parametro, False)
                if excecao:
                    max_age = 0
            headers['Cache-Control'] = "max-age={}".format(max_age)



# -*- coding: utf-8 -*-

"""
Funções para auxiliarem no desenvolvimento
"""

import time
from functools import wraps


def tente_outra_vez(excecoes, tentativas=4, tempo_espera=3, multiplicador_espera=1, logger=None):
    """
    Escuta pelas exceções especificadas e refaz a chamada para o método.
    :param excecoes: As exceções que devem ser escutadas
    :type excecoes: Exception or tuple
    :param tentativas: Número de vezes que deve ser chamado o método
    :type tentativas: int
    :param tempo_espera: Tempo inicial de espera em segundos entre as tentativas
    :type tempo_espera: int
    :param multiplicador_espera: Multiplicador para o tempo de espera nas chamadas subsequentes.
    :type multiplicador_espera: int
    :param logger: Mecanismo de log
    :type logger: logging.Logger instance
    """
    def deco_tente_outra_vez(funcao):
        """
        Decorator para tente_outra_vez
        """
        @wraps(funcao)
        def funcao_tente_outra_vez(*args, **kwargs):
            """
            Tratamento de decorator para tente_outra_vez
            """
            _tentativas, _tempo_espera = tentativas, tempo_espera
            while _tentativas > 1:
                try:
                    return funcao(*args, **kwargs)
                except excecoes, exc:
                    msg = "{}, Reconectando in {} segundos...".format(str(exc), _tempo_espera)
                    if logger:
                        logger.warning(msg)
                    time.sleep(_tempo_espera)
                    _tentativas -= 1
                    _tempo_espera *= multiplicador_espera
            return funcao(*args, **kwargs)
        return funcao_tente_outra_vez
    return deco_tente_outra_vez



# -*- coding: utf-8 -*-

"""
Regras e facilitadores para formatação de dados
"""
import json
import traceback

from urllib import quote_plus
from unicodedata import normalize
from xml.etree import ElementTree
import decimal
from xml.etree.ElementTree import ParseError
import sys


class Formatador(object):
    """
    Classe a ser injetada na entidade que precisa de recursos de formatação
    """
    @classmethod
    def formata_data(cls, data, hora=True, iso=False):
        """
        Formata data nos padrões AAAA-MM-DD HH:mm:ss, AAAA-MM-DDTHH:mm:ss ou AAAA-MM-DD
        :param data: A data a ser formatada
        :type data: datetime
        :param hora: Define se a hora deve vir na data formatada.
        :type hora: bool
        :param iso: Define se a hora virá no formato ISO
        :type iso: bool
        :return: A data formatada de acordo com os parâmetros passados
        :rtype: str
        """
        if hora:
            return data.strftime("%Y-%m-%d{}%H:%M:%S".format("T" if iso else " "))
        return data.strftime("%Y-%m-%d")

    @classmethod
    def formata_cpf(cls, cpf):
        """
        Formata o CPF no padrão XXX.XXX.XXX-XX
        :param cpf: O número do CPF a ser formatado.
        :type cpf: str
        :return: O CPF formatado ou None se o valor for inválido para um CPF
        :rtype: str
        """
        cpf = cpf.replace('.', '').replace('-', '')
        if len(cpf) == 11 and cpf.isdigit():
            return '%s.%s.%s-%s' % (cpf[:3], cpf[3:6], cpf[6:9], cpf[9:])
        return None

    @classmethod
    def formata_cnpj(cls, cnpj):
        """
        Formata o CNPJ no padrão XX.XXX.XXX/XXXX-XX
        :param cnpj: O número do CNPJ a ser formatado.
        :type cnpj: str
        :return: O CNPJ formatado ou None se o valor for inválido para um CNPJ
        :rtype: str
        """
        cnpj = cnpj.replace('.', '').replace('-', '').replace('/', '')
        if len(cnpj) == 14 and cnpj.isdigit():
            return '%s.%s.%s/%s-%s' % (cnpj[:2], cnpj[2:5], cnpj[5:8], cnpj[8:12], cnpj[12:])
        return None

    @classmethod
    def formata_cpf_cnpj(cls, cpf_cnpj):
        """
        Aceita tanto um CPF quanto um CNPJ e retorna o mesmo formatado de acordo com o tamanho do valor enviado.
        :param cpf_cnpj: O número a ser formatado. Se com 11 caracters, formata como CPF, se 14, como CNPJ.
        :type cpf_cnpj: str
        :return: O documento formatado conforme o tamanho.
        :rtype: str
        """
        if not cpf_cnpj:
            return ''
        formatado = cls.formata_cpf(cpf_cnpj) or cls.formata_cnpj(cpf_cnpj)
        if not formatado:
            return cpf_cnpj
        return formatado

    @classmethod
    def string_para_ascii(cls, texto):
        """
        Formata um texto para ascii, removendo caracteres especiais e convertendo acentuados para não acentuados
        :param texto: O texto a ser formatado.
        :type texto: str
        :return: O texto so com caracters ascii
        :rtype: str
        """
        try:
            return normalize('NFKD', texto.decode('utf-8')).encode('ASCII', 'ignore')
        except UnicodeEncodeError:
            return normalize('NFKD', texto).encode('ASCII', 'ignore')

    @classmethod
    def string_para_url(cls, texto):
        """
        Formata a string para ser usada em uma URK (URL Encode)
        :param texto: O texto a ser formatado.
        :type texto: str
        :return: O texto formatado com URL Encode.
        :rtype: str
        """
        return quote_plus(unicode(texto))

    @classmethod
    def converte_para_decimal(cls, valor):
        """
        Converte um valor para Decimal. Caso não seja possível converter, retorna Decimal('0.00')
        :param valor: O valor a ser convertido, podendo ser str, float ou bool
        :return: O valor convertido para Decimal
        :rtype: decimal.Decimal
        """
        try:
            return decimal.Decimal(valor)
        except (decimal.InvalidOperation, ValueError, TypeError):
            return decimal.Decimal('0.00')

    @classmethod
    def trata_unicode_com_limite(cls, texto, limite=None, ascii=False, trata_espaco_duplo=False):
        """
        Trata texto unicode e retorna de acordo com os parâmetros.
        :param texto: O texto a ser tratado.
        :type texto: unicode
        :param limite: Define q quantidade de caracteres a ser retornado de texto.
        :type limite: int
        :param ascii: Define se o texto deve ser retornado apenas com caracteres ascii
        :type ascii: bool
        :param trata_espaco_duplo: Define se espaços duplos entre palavras devem ser trocados por um espaço apenas.
         :type trata_espaco_duplo: bool
        :return: O texto codificado em utf-8 ou ascii conforme o caso
        :rtype: str
        """
        if texto is None:
            return ""
        if isinstance(texto, unicode):
            texto = texto.encode("utf-8")
        if ascii:
            texto = cls.string_para_ascii(texto)
        if trata_espaco_duplo:
            texto = " ".join([palavra for palavra in texto.split(" ") if palavra.strip()])
        if limite:
            return texto[:limite]
        return texto

    @classmethod
    def trata_nome(cls, nome):
        """
        Trata caso especial de nome mal formatado, garantindo que o mesmo tenha pelo ao menos nome e sobrenome e retirando caracteres que podem conflitar com formatação de URL como & e ?.
        :param nome: O nome a ser formatado.
        :type nome: str
        :return: O nome com os ajustes necessários.
        :rtype: str
        """
        if nome:
            nome = nome.strip()
        if len(nome.split(" ")) < 2:
            nome = u"{} x".format(nome)
        if "&" in nome:
            nome = nome.replace("&", "E")
        if "?" in nome:
            nome = nome.replace("?", " ")
        return nome

    @classmethod
    def formata_decimal(cls, valor, como_float=False, como_int=False, em_centavos=False):
        """
        Formata decimal de acordo com os parâmetros passado. O padrão é retornar como string com duas casas decimais.
        :param valor: O valor a ser formatado.
        :type valor: decimal.Decimal, float ou int
        :param como_float: Define se o valor deve ser retornado como float ao invés de string
        :type como_float: bool
        :param como_int: Define se o valor deve ser retornado como int ao invés de string
        :type como_int: bool
        :param em_centavos: Define se o valor deve ser multiplicado por 100 e retornado como um inteiro.
        :type em_centavos: bool
        :return: O valor formatado como string ou de acordo com os parâmetros
        :rtype: str
        """
        if como_float:
            return float('{0:.2f}'.format(valor))
        if como_int:
            return int(valor)
        if em_centavos:
            return int(valor * 100)
        return '{0:.2f}'.format(valor)

    @classmethod
    def converte_tel_em_tupla_com_ddd(cls, telefone):
        """
        Converte um número de telefone em tupla com (DDD, Número). O DDD deve ser os dois primeiros caracteres do telefone passado.
        O método valida se o DDD é um número válido e pertence a algum DDD do Brasil.
        Também valida se o telefone tem 8 ou 9 dígitos.
        :param telefone: Número de telefone com DDD.
        :type telefone: str
        :return: Uma tupla contento o DDD e o Número do telefone.
        :rtype: tuple
        """
        ddd = telefone[:2]
        numero = telefone[2:]
        if not ddd.isdigit() or not telefone.isdigit():
            return '', ''
        if int(ddd) < 11:
            return '', ''
        ddds_invalidos = [20, 23, 25, 26, 29, 30, 36, 39, 40, 50, 52, 56, 57, 58, 59, 60, 70, 72, 76, 78, 80, 90]
        if int(ddd) in ddds_invalidos:
            return '', ''
        if len(numero) < 8 or len(numero) > 9:
            return '', ''
        if numero.startswith("0"):
            return '', ''
        return ddd, numero

    @classmethod
    def trata_email_com_mais(cls, email):
        """
        Remove a parte +algumacoisa da parte caixa postal de um e-mail. Ex.: email+algumacoisa@email.com -> email@email.com
        :param email: O email a ser tratado
        :type email: str
        :return: O email sem o +xxx s existir.
        :rtype: str
        """
        if "+" not in email:
            return email
        partes = email.split("@")
        caixa = partes[0].split("+")[0]
        return "{}@{}".format(caixa, partes[1])

    @classmethod
    def dict_para_xml(cls, dados, tem_cabecalho=False):
        """
        Converte um dicionário em xml.
        Consegue converter subníveis e elementos como lista.
        :param dados: Os dados a serem convertidos em xml
        :type dados: dict
        :param tem_cabecalho: Informa que o xml já possui cabeçalho e não precisa incluir. Se esse valor for False, o método irá adiocionar um <?xml version="1.0" encoding="UTF-8" standalone="yes"?> no início.
        :type tem_cabecalho: bool
        :return: Uma string contendo os valores do dicionário como xml
        :rtype: str
        """
        if not isinstance(dados, dict):
            return ""
        if not dados:
            return ""
        documento = []
        if not tem_cabecalho:
            documento.append('<?xml version="1.0" encoding="UTF-8" standalone="yes"?>')
        for chave, valor in dados.iteritems():
            documento.append("<{}>".format(chave))
            if isinstance(valor, dict):
                documento.append(cls.dict_para_xml(valor, True))
            elif isinstance(valor, list):
                for parte in valor:
                    documento.append(cls.dict_para_xml(parte, True))
            else:
                documento.append(unicode(valor))
            documento.append("</{}>".format(chave))
        return "".join(documento)

    @classmethod
    def xml_para_dict(cls, content):
        """
        Converte uma string contendo um XML válido em um dicionário python. Caso não seja possível ler o XML, retorna um dicionário vazio.
        Consegue converter subníveis e elementos em lista.
        :param content: Uma string com um XML válido.
        :type content: str
        :return: Um dicionário contendo os valores do XML.
        :rtype: dict
        """
        def iterando(elemento):
            """
            Itera pelo elemento XML e converte para dicionário
            :param elemento: Um elemento xml
            :type elemento: xml.etree.ElementTree
            :return: O elemento como dicionário
            :rtype: dict
            """
            resultado = {}
            childrens = elemento.getchildren()
            if len(childrens) > 0:
                if len(elemento.findall(childrens[0].tag)) > 1:
                    resultado[elemento.tag] = []
                else:
                    resultado[elemento.tag] = {}
                for children in childrens:
                    if isinstance(resultado[elemento.tag], list):
                        resultado[elemento.tag].append(iterando(children))
                    else:
                        resultado[elemento.tag].update(iterando(children))
            else:
                resultado[elemento.tag] = elemento.text
            return resultado

        try:
            root = ElementTree.fromstring(content)
            return iterando(root)
        except (ParseError, TypeError):
            return {}


class ResultadoDeApi(object):
    """
    Fornece métodos para padronizar a serialização de respostas das APIs
    """
    _resultados = {
        200: 'sucesso',
        400: 'request_invalido',
        401: 'nao_autorizado',
        403: 'nao_autenticado',
        404: 'nao_encontrado',
        405: 'metodo_nao_permitido',
        408: 'timeout',
        500: 'erro_servidor'
    }

    @classmethod
    def _formata_trace_back(cls, trace_back):
        """
        Formata o trace back como uma lista de dicionários no seguinte formato:
        {
            'codigo': '    a = 1 / 0',
            'local': '..unitarios/padroes/test_serializacao.py, line 318, in test_deve_retornar_resposta_para_erro_500_com_trace_back_passado'
        }
        :param trace_back: O traceback da exceção
        :type trace_back: list
        :return: A lista de traceback formatada
        :rtype: list
        """
        resultado = []
        for stack in trace_back[-4:-1]:
            linhas = stack.split('\n')
            local = linhas[0].split(', ')
            local[0] = local[0].strip()
            local[0] = local[0].replace('File "', '').replace('"', '')
            local[0] = '..{}'.format('/'.join(local[0].split('/')[-3:]))
            local = ", ".join(local)
            if not local.startswith('..Traceback '):
                resultado.append({'local': local, 'codigo': linhas[1]})
        return resultado

    @classmethod
    def resposta(cls, conteudo, nome_api, versao, status=200, excecao=None):
        """
        Monta a resposta para os dados do resultado.
        :param conteudo: Um dicionário com o conteúdo da resposta a ser enviado. Caso não seja passado um dicionário, o mesmo será criado com a chave 'conteudo' e o valor passado nesse parametro.
        :type conteudo: dict
        :param nome_api: O nome da API que está produzindo a resposta
        :type nome_api: str
        :param versao: A versão da API que está produzindo a resposta
        :type versao: str
        :param status: O status code da resposta
        :type status: int
        :param excecao: A exceção que gerou uma resposta com status code 500
        :type excecao: Exception
        :return: Tupla com os dados do resultado e o status code
        :rtype: tuple
        """
        if status == 403:
            conteudo = {'conteudo': u'A autenticação falhou: {}'.format(conteudo)}
        if status == 404:
            conteudo = {'conteudo': u'A url acessada não existe em nome-da-api'}
        if status == 405:
            conteudo = {'conteudo': u'Não é permitido esse método HTTP para essa URL.'}
        if status == 408:
            conteudo = {'conteudo': u'O servidor não respondeu em tempo.'}
        if not isinstance(conteudo, dict):
            conteudo = {'conteudo': conteudo}
        try:
            status_resultado = cls._resultados[status]
        except KeyError:
            status_resultado = 'status_nao_definido'
        metadados = {
            'api': nome_api,
            'versao': versao,
            'resultado': status_resultado,
        }
        conteudo = conteudo
        if excecao:
            exc_type, exc_value, trace_back = sys.exc_info()
            if exc_type:
                stack_trace = traceback.format_exception(exc_type, exc_value, trace_back)
            else:
                try:
                    stack_trace = traceback.format_exception(excecao.tipo, excecao, excecao.tb)
                except AttributeError:
                    stack_trace = []
            stack_trace = cls._formata_trace_back(stack_trace)
            nome_exec = excecao.__class__.__name__
            conteudo.update({'excecao': {'nome': nome_exec, 'mensagem': unicode(excecao), 'stack_trace': stack_trace}})
        resultado = {
            'metadados': metadados,
            status_resultado: conteudo
        }
        return json.dumps(resultado), status



# -*- coding: utf-8 -*-

"""
Componentes para realizar chamadas HTTPs
"""

import json
import urllib
import requests
from requests.exceptions import ConnectionError, Timeout
from li_common import helpers
from li_common.padroes import serializacao


REQUEST_BASE_TIMEOUT = (10, 20)
GRAVA_EVIDENCIA = False


class TipoMetodo(object):
    """
    Enumerador para tipos de métodos HTTP
    """
    get = "get"
    post = "post"
    put = "put"
    delete = "delete"


class Formato(object):
    """
    Enumerador para tipos de formato e o Content-Type equivalente.
    """
    texto = 'text'
    querystring = 'text/html'
    json = 'application/json'
    form_urlencode = 'application/x-www-form-urlencoded'
    xml = 'application/xml'


class Credenciador(object):
    """
    Serviço padrão para construir credenciadores de conexão HTTP para ser usado em li_common.comunicacao.requisicao.Conexao
    """
    class TipoAutenticacao(object):
        """
        Enumerador para tipos de autenticação.
        """
        cabecalho_http = 1
        query_string = 2
        form_urlencode = 3

    def __init__(self, tipo=TipoAutenticacao.cabecalho_http, chave=None):
        self.tipo = tipo
        self.chave = chave or 'Authorization' if tipo == self.TipoAutenticacao.cabecalho_http else 'access_token'

    def define_autenticacao(self):
        """
        Define os dados de autenticação paseado no tipo.
        :return O resultado de ums dos métodos de obtenção de dados de acordo com o tipo de autenticação.
        """
        if self.tipo == self.TipoAutenticacao.cabecalho_http:
            return self.por_cabecalho_http()
        if self.tipo == self.TipoAutenticacao.form_urlencode:
            return self.por_form_urlencoded()
        if self.tipo == self.TipoAutenticacao.query_string:
            return self.por_query_string()
        return None

    def obter_credenciais(self):
        """
        Monta os dados da autenticação
        :return Os valores da autenticação para serem usados por um dos métodos de formatação de acordo com o tipo de autenticação.
        """
        return ''

    def por_query_string(self):
        """
        Formata os dados da autenticação em um par chave=valor
        """
        return "{}={}".format(self.chave, self.obter_credenciais())

    def por_form_urlencoded(self):
        """
        Formata os dados da autenticação em um dicionário
        """
        return {self.chave: self.obter_credenciais()}

    def por_cabecalho_http(self):
        """
        Formata os dados da autenticação em um dicionário
        """
        return {self.chave: self.obter_credenciais()}


class RespostaJsonInvalida(Exception):
    """
    Disparada quando o formato de resposta é definido como JSON porem o conteúdo da resposta não pode ser codificado em JSON.
    """
    pass


class Resposta(object):
    """
    Classe que encapsula os dados de u request.response e já formata o conteúdo em dicionário.
    """
    def __init__(self, request_response, formato_resposta=Formato.json):
        self.sucesso = request_response.status_code in (200, 201)
        self.requisicao_invalida = request_response.status_code == 400
        self.nao_autorizado = request_response.status_code == 401
        self.nao_autenticado = request_response.status_code == 403
        self.nao_encontrado = request_response.status_code == 404
        self.timeout = request_response.status_code == 408
        self.erro_servidor = request_response.status_code == 500
        self.status_code = request_response.status_code
        self.conteudo = {}
        if formato_resposta == Formato.xml:
            self.conteudo = serializacao.Formatador.xml_para_dict(request_response.content)
        if formato_resposta == Formato.json:
            try:
                self.conteudo = json.loads(request_response.content)
            except ValueError:
                raise RespostaJsonInvalida(u'O conteudo "{}" não pode ser transformado em JSON.'.format(request_response.content))
        if formato_resposta in (Formato.form_urlencode, Formato.querystring):
            content = urllib.unquote(request_response.content).decode('utf8')
            self.conteudo = {par.split("=")[0].lower(): par.split("=")[1].lower() for par in content.split("&")}
        if formato_resposta == Formato.texto:
            self.conteudo = request_response.content


class Conexao(object):
    """
    Componente para realizar a conexão HTTP.
    """
    def __init__(self, formato_envio=Formato.json, formato_resposta=Formato.json, headers=None, credenciador=None):
        self.headers = {'Content-Type': '{}; charset=ISO-8859-1'.format(formato_envio)}
        self.formato_envio = formato_envio
        self.formato_resposta = formato_resposta
        self.credenciador = credenciador
        if headers:
            self.headers.update(headers)
        self.dados_envio = {}

    def define_url_com_autenticacao(self, url):
        """
        Define a autenticação a ser usada.
        Se for do tipo cabeçalho http, adiciona o resultado do credenciador no header.
        Se for do tipo form urlencode, adiciona o resultado do credenciador nos dados a serem enviados.
        Se for do tipo query string, adiciona o resultado do credenciador na url passada.
        :param url: A url que será chamada, onde a autenticação será adicionada caso o tipo de envio seja por query string.
        :type url: str
        :return: A url formatada
        :rtype: str
        """
        autenticacao = self.credenciador.define_autenticacao()
        if self.credenciador.tipo == Credenciador.TipoAutenticacao.cabecalho_http:
            self.headers.update(autenticacao)
        if self.credenciador.tipo == Credenciador.TipoAutenticacao.form_urlencode:
            self.dados_envio.update(autenticacao)
        if self.credenciador.tipo == Credenciador.TipoAutenticacao.query_string:
            separador = "&" if "?" in url else "?"
            return "{}{}{}".format(url, separador, autenticacao)
        return url

    @helpers.tente_outra_vez((ConnectionError, Timeout), tentativas=3, tempo_espera=4)
    def faz_request(self, url, metodo=TipoMetodo.get):
        """
        Executa o método HTTP especificado na url
        :param url: A url onde será feita a chamada.
        :type url: str
        :param metodo: Um dos métodos HTTP disponíveis
        :type metodo: str
        :return: Um objeto Resposta com os dados retornados pelo request.response.
        :rtype: Resposta
        """
        if self.credenciador:
            url = self.define_url_com_autenticacao(url)
        if self.formato_envio == Formato.querystring:
            resposta = getattr(requests, metodo)(url, params=self.dados_envio, headers=self.headers, timeout=REQUEST_BASE_TIMEOUT)
        elif self.formato_envio == Formato.form_urlencode:
            resposta = getattr(requests, metodo)(url, data=self.dados_envio, headers=self.headers, timeout=REQUEST_BASE_TIMEOUT)
        elif self.formato_envio == Formato.xml:
            dados = self.dados_envio.get('dados', '')
            resposta = getattr(requests, metodo)(url, data=dados, headers=self.headers, timeout=REQUEST_BASE_TIMEOUT)
        else:
            dados_envio = self.dados_envio
            if type(self.dados_envio is dict):
                dados_envio = json.dumps(self.dados_envio)
            resposta = getattr(requests, metodo)(url, data=dados_envio, headers=self.headers, timeout=REQUEST_BASE_TIMEOUT)
        return Resposta(resposta, formato_resposta=self.formato_resposta)

    def prepara_request(self, url, metodo=TipoMetodo.get, dados=None):
        """
        Método de preparação para o request.
        :param url: A url onde será feita a chamada.
        :type url: str
        :param metodo: Um dos métodos HTTP disponíveis
        :type metodo: str
        :param dados: Dados a serem passados para o request.
        :type dados: dict or str
        :return: Um objeto Resposta com os dados retornados pelo request.response.
        :rtype: Resposta
        """
        if dados:
            if isinstance(dados, dict):
                self.dados_envio.update(dados)
            else:
                self.dados_envio["dados"] = dados
        resposta = self.faz_request(url, metodo)
        return resposta

    def get(self, url, dados=None):
        """
        Realiza uma chamada GET na url com os dados passados
        :param url: A url onde será feita a chamada.
        :type url: str
        :param dados: Dados a serem passados para o request.
        :type dados: dict or str
        :return: Um objeto Resposta com os dados retornados pelo request.response.
        :rtype: Resposta
        """
        return self.prepara_request(url, TipoMetodo.get, dados)

    def post(self, url, dados=None):
        """
        Realiza uma chamada POST na url com os dados passados
        :param url: A url onde será feita a chamada.
        :type url: str
        :param dados: Dados a serem passados para o request.
        :type dados: dict or str
        :return: Um objeto Resposta com os dados retornados pelo request.response.
        :rtype: Resposta
        """
        return self.prepara_request(url, TipoMetodo.post, dados)

    def put(self, url, dados=None):
        """
        Realiza uma chamada PUT na url com os dados passados
        :param url: A url onde será feita a chamada.
        :type url: str
        :param dados: Dados a serem passados para o request.
        :type dados: dict or str
        :return: Um objeto Resposta com os dados retornados pelo request.response.
        :rtype: Resposta
        """
        return self.prepara_request(url, TipoMetodo.put, dados)

    def delete(self, url, dados=None):
        """
        Realiza uma chamada DELETE na url com os dados passados
        :param url: A url onde será feita a chamada.
        :type url: str
        :param dados: Dados a serem passados para o request.
        :type dados: dict or str
        :return: Um objeto Resposta com os dados retornados pelo request.response.
        :rtype: Resposta
        """
        return self.prepara_request(url, TipoMetodo.delete, dados)



# -*- coding: utf-8 -*-

"""
Componentes para a criação de recursos para serem usados nas APIs Rest
"""
from flask import make_response, request
import flask_restful
from werkzeug import exceptions

from li_common.padroes import serializacao


class RecursoBase(flask_restful.Resource):
    """
    Representa um recurso base de api rest
    """
    app = None

    def __init__(self):
        self._dados = {}

    @property
    def dados(self):
        """
        Obtem os dados enviados em um request, analisando args, form e json e adicionando todos em um dicionário único
        :return: Um dicionário com os valores dos dados passados no request
        :rtype: dict
        """
        if not self._dados:
            self._dados = {}
            for arg in request.args:
                self._dados[arg] = request.args[arg]
            for arg in request.form:
                self._dados[arg] = request.form[arg]
            try:
                self._dados.update(request.get_json())
            except (TypeError, exceptions.BadRequest):
                pass
        return self._dados

    @classmethod
    def resposta(cls, conteudo, status=200, excecao=None):
        """
        Gera um objeto response do Flask com um conteúdo padronizado para todos os tipos de retorno
        :param conteudo: O conteudo a ser inserido no padrão de resposta. Pode ser um dict ou uma str
        :type conteudo: dict, str
        :param status: Status Code para a resposta. Padrão é 200
        :type status: int
        :param excecao: Caso a resposta seja 500, qual a exceção que gerou o erro 500.
        :type excecao: Exception
        :return: Um objeto response do Flask
        """
        conteudo, status = serializacao.ResultadoDeApi.resposta(conteudo, cls.app.nome_api, cls.app.version, status, excecao)
        headers = {'Content-Type': 'application/json; charset=utf-8'}
        cls.app.cache.aplica_cache(request, headers)
        return make_response(conteudo, status, headers)
